
#from click.testing import CliRunner

#from knox import cli


def test_main():
    pass
#    runner = CliRunner()
#    result = runner.invoke(main, [])
#
#    assert result.output == '()\n'
#    assert result.exit_code == 0
