"""
Apache Software License 2.0

Copyright (c) 2020, 8x8, Inc.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

https://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License."""
from . import backend as backend
from . import config as config


class Knox:
    """Composite class for Knox package"""
    _conf: config.Conf
    _store: backend.Store
    _cli: dict

    def __init__(self, loglevel=None) -> None:
        """Constructor for Knox """

        self._conf = config.Conf(loglevel)
        self._store = backend.Store(self._conf.settings)

    @property
    def settings(self) -> config.Conf.settings:
        """Access to the Dynaconf settings object"""
        return self._conf.settings

    @property
    def conf(self) -> config.Conf:
        """Access to the knox Conf object"""
        return self._conf

    @property
    def store(self) -> backend.Store:
        """Access to the instantiated store engine"""
        return self._store
